package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.dao.*;
import com.ims.eval.entity.CalculateIndicatorItemInfo;
import com.ims.eval.entity.OrganizationEvaluationInfo;
import com.ims.eval.entity.dto.response.OrganizationEvaluationResDTO;
import com.ims.eval.service.ICalculateIndicatorItemInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 计算指标项明细 服务实现类
 * </p>
 *
 * @author xugp
 * @since 2023-03-26
 */
@Service
public class CalculateIndicatorItemInfoServiceImpl extends ServiceImpl<CalculateIndicatorItemInfoMapper, CalculateIndicatorItemInfo> implements ICalculateIndicatorItemInfoService {



	@Autowired
	private OrganizationEvaluationMapper organizationEvaluationMapper;

	@Autowired
	private CalculateIndicatorItemInfoMapper calculateIndicatorItemInfoMapper;

	@Autowired
	private IndicatorMapper indicatorMapper;


	@Override
	public int saveIndicatorInfo(List<OrganizationEvaluationInfo> organizationEvaluationInfos) throws Exception {

		List<CalculateIndicatorItemInfo> list = new ArrayList<>();

		OrganizationEvaluationResDTO organizationEvaluationResDTO = organizationEvaluationMapper.selectById(organizationEvaluationInfos.get(0).getOrganizationEvaluationId());
		//对考评规则id进行分组
		Map<String, List<OrganizationEvaluationInfo>> collect = organizationEvaluationInfos.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getOrganizationEvaluationRuleId));
		for (Map.Entry<String, List<OrganizationEvaluationInfo>> entry : collect.entrySet()) {
			if (entry.getValue().size() > 0) {
				//对指标id进行分组
				Map<String, List<OrganizationEvaluationInfo>> collectIn = entry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getIndicatorId));
				for (Map.Entry<String, List<OrganizationEvaluationInfo>> listEntry : collectIn.entrySet()) {
					List<OrganizationEvaluationInfo> listEntryValue = listEntry.getValue();
					for (OrganizationEvaluationInfo r : listEntryValue){
						if (r.getOptionCode().equals("CZ") || r.getOptionCode().equals("LRGXL") || r.getOptionCode().equals("DWQWLR")) {
							CalculateIndicatorItemInfo calculateIndicatorItemInfo = new CalculateIndicatorItemInfo();
							calculateIndicatorItemInfo.setIndicatorId(listEntry.getKey());
							calculateIndicatorItemInfo.setOptionCode(r.getOptionCode());
							calculateIndicatorItemInfo.setQuantifiedValue(r.getQuantifiedValue());
							calculateIndicatorItemInfo.setSectionId(indicatorMapper.selectById(listEntry.getKey()).getBinSection());
							calculateIndicatorItemInfo.setOrganizationType(organizationEvaluationResDTO.getOrganizationType());
							calculateIndicatorItemInfo.setCheckCycle(organizationEvaluationResDTO.getCheckCycle());
							calculateIndicatorItemInfo.setYear(organizationEvaluationResDTO.getYear());
							calculateIndicatorItemInfo.setMonth(organizationEvaluationResDTO.getMonth());
							list.add(calculateIndicatorItemInfo);
						}
					}
				}
			}
		}

		return saveCalculate(list);
	}


	@Transactional
	public int saveCalculate(List<CalculateIndicatorItemInfo> list) throws Exception {

		//遍历list解析最大值和最小值
		if (list.size() > 0){
			Map<String, List<CalculateIndicatorItemInfo>> collect = list.stream().collect(Collectors.groupingBy(CalculateIndicatorItemInfo::getOptionCode));
			for (Map.Entry<String, List<CalculateIndicatorItemInfo>> listEntry : collect.entrySet()){

				//从小到大的顺序排序
				Collections.sort(listEntry.getValue(), new Comparator<CalculateIndicatorItemInfo>() {
					@Override
					public int compare(CalculateIndicatorItemInfo u1, CalculateIndicatorItemInfo u2) {
						double diff = u1.getQuantifiedValue() - u2.getQuantifiedValue();
						if (diff > 0) {
							return 1;
						} else if (diff < 0) {
							return -1;
						}
						return 0; //相等为0
					}
				});

				CalculateIndicatorItemInfo maxNum = listEntry.getValue().get(listEntry.getValue().size()-1);
				maxNum.setMark("1");

				CalculateIndicatorItemInfo minNum = listEntry.getValue().get(0);
				minNum.setMark("0");

				//判断计算指标明细中之前是否存在记录 如果存在删除之前记录,重新新增
				QueryWrapper<CalculateIndicatorItemInfo> qw = new QueryWrapper<>();
				qw.lambda().eq(CalculateIndicatorItemInfo::getYear, maxNum.getYear());
				qw.lambda().eq(CalculateIndicatorItemInfo::getMonth, maxNum.getMonth());
				qw.lambda().eq(CalculateIndicatorItemInfo::getOptionCode, maxNum.getOptionCode());
				qw.lambda().eq(CalculateIndicatorItemInfo::getCheckCycle, maxNum.getCheckCycle());
				qw.lambda().eq(CalculateIndicatorItemInfo::getOrganizationType, maxNum.getOrganizationType());
				qw.lambda().eq(CalculateIndicatorItemInfo::getMark, maxNum.getMark());
				CalculateIndicatorItemInfo calculateIndicatorItemInfo = calculateIndicatorItemInfoMapper.selectOne(qw);

				if (calculateIndicatorItemInfo != null){
					calculateIndicatorItemInfoMapper.deleteById(calculateIndicatorItemInfo.getId());
				}

				if (listEntry.getKey().equals("CZ")){
					//存储最大值
					maxNum.setOptionCode("CZMAX");
				}else if (listEntry.getKey().equals("LRGXL")){
					maxNum.setOptionCode("LRGXKLMAX");
				}else if (listEntry.getKey().equals("DWQWLR")){
					maxNum.setOptionCode("DWQWLRMAX");
				}
				int insert = calculateIndicatorItemInfoMapper.insert(maxNum);

				//判断计算指标明细中之前是否存在记录 如果存在删除之前记录,重新新增
				QueryWrapper<CalculateIndicatorItemInfo> qw1 = new QueryWrapper<>();
				qw1.lambda().eq(CalculateIndicatorItemInfo::getYear, minNum.getYear());
				qw1.lambda().eq(CalculateIndicatorItemInfo::getMonth, minNum.getMonth());
				qw1.lambda().eq(CalculateIndicatorItemInfo::getOptionCode, minNum.getOptionCode());
				qw1.lambda().eq(CalculateIndicatorItemInfo::getCheckCycle, minNum.getCheckCycle());
				qw1.lambda().eq(CalculateIndicatorItemInfo::getOrganizationType, minNum.getOrganizationType());
				qw1.lambda().eq(CalculateIndicatorItemInfo::getMark, minNum.getMark());
				CalculateIndicatorItemInfo calculateIndicatorItemInfo1 = calculateIndicatorItemInfoMapper.selectOne(qw1);

				if (calculateIndicatorItemInfo1 != null){
					calculateIndicatorItemInfoMapper.deleteById(calculateIndicatorItemInfo1.getId());
				}

				if (listEntry.getKey().equals("CZ")){
					//存储最小值
					minNum.setOptionCode("CZMIN");
				}else if (listEntry.getKey().equals("LRGXL")){
					//存储最小值
					minNum.setOptionCode("LRGXKLMIN");
				}else if (listEntry.getKey().equals("DWQWLR")){
					//存储最小值
					minNum.setOptionCode("DWQWLRMIN");
				}
				int insert1 = calculateIndicatorItemInfoMapper.insert(minNum);
			}
		}else {
			throw new Exception("计算指标项明细为空");
		}
		return 1;
	}
}
